// Configuration
// #define F_CPU		 4000000
// #define F_CPU		 8000000
#define F_CPU			16000000
#define UART_BAUD_RATE		  9600
// #define UART_BAUD_RATE	 19200
// #define UART_BAUD_RATE	 38400
// #define UART_BAUD_RATE	 57600
// #define UART_BAUD_RATE	115200

#include <string.h>
#include <stdlib.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

#include "pwm.c"
#include "serial.c"
#include "delay.c"
#include "timer.c"
#include "stepper.c"
#include "motor.c"


long atolong(char *str) {
	int n = 0;
	int str_len = strlen(str);
	long value = 0;
	for (n = 0; n < str_len; n++) {
		if (str[n] < 48 || str[n] > 58) {
			break;
		}
		value *= 10;
		value += (str[n] - 48);
	}
	return value;
}

int main(void) {
	char *loc1;
	char *loc2;
	unsigned long n;
	unsigned long position[6];
	unsigned long sposition[6];
	unsigned long diff[6];
	unsigned long max_diff = 0;
	unsigned char direction[6];
	unsigned char axis_n = 0;
	unsigned char min_speed = 255;
	unsigned char max_soft = 0;
	unsigned char max_start_delay = 0;
	unsigned int  max_delay = 0;
	unsigned int  max_delay2 = 0;
	unsigned int cmd_len = 0;
	unsigned int cmd_sum = 0;
	unsigned int cmd_org = 0;
	unsigned char soft_start = 0;
	Uart_Init();
	stepper_init();
	motor_init();
	Uart_Send_Str("mXm CNC-Interface v0.5\n");
	for (;;) {
		if (receive_flag == 2) {
			stepper_input();
			cmd_len = strlen(SerialBuffer);
			cmd_sum = 0;
			cmd_org = SerialBuffer[cmd_len - 1];
			SerialBuffer[cmd_len - 1] = 0;
			for (n = 0; n < cmd_len - 1; n++) {
				cmd_sum += SerialBuffer[n];
			}
			cmd_sum = cmd_sum / (cmd_len - 1) + (cmd_len - 1);
			if (cmd_org != cmd_sum) {
				Uart_Send_Str("CHKSUM_ERROR: ");
				Uart_Send_Int(cmd_org);
				Uart_Send_Str(" != ");
				Uart_Send_Int(cmd_sum);
				Uart_Send_Str("\n");
			} else {
				if (SerialBuffer[0] == 'P') {
					stepper_pos(SerialBuffer[1] - 48, SerialBuffer[2], atolong(SerialBuffer + 3), 0);
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'S' && SerialBuffer[2] == '=') {
					stepper_speed(SerialBuffer[1] - 48, atoi(SerialBuffer + 3));
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'D' && SerialBuffer[2] == '=') {
					stepper_delay(SerialBuffer[1] - 48, atoi(SerialBuffer + 3));
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'I') {
					stepper_info(SerialBuffer[1] - 48);
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'F' && SerialBuffer[1] == '=') {
					soft_start = atoi(SerialBuffer + 2);
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'W') {
					axis_n = SerialBuffer[1] - 48;
					stepper_speed(axis_n, 255);
					while(!(Axis[axis_n].state & (1<<STAT_SW2))) {
						stepper_pos(axis_n, '+', 1, 0);
						delay_ms(1);
					}
					Uart_Send_Int(Axis[axis_n].position);
					Uart_Send_Str("\n");
					stepper_pos(axis_n, '=', 0, 0);
				} else if (SerialBuffer[0] == 'M') {
					motor_set(SerialBuffer[1] - 48, atoi(SerialBuffer + 3));
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'R') {
					axis_n = SerialBuffer[1] - 48;
					stepper_speed(axis_n, 255);
					if (axis_n < 4) {
						while(!(Axis[axis_n].state & (1<<STAT_SW1))) {
							stepper_pos(axis_n, '-', 1, 0);
						}
					} else {
						stepper_pos(axis_n, '=', 0, 0);
					}
					Axis[axis_n].position = 0;
					Axis[axis_n].speed = 255;
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'N') {
					axis_n = SerialBuffer[1] - 48;
					if (SerialBuffer[2] == '+') {
						Axis[axis_n].position += atolong(SerialBuffer + 3);
					} else if (SerialBuffer[2] == '-') {
						Axis[axis_n].position -= atolong(SerialBuffer + 3);
					}
					Uart_Send_Str("OK\n");
				} else if (SerialBuffer[0] == 'A') {
					for (axis_n = 0; axis_n < 6; axis_n++) {
						position[axis_n] = Axis[axis_n].position;
						sposition[axis_n] = Axis[axis_n].position;
						direction[axis_n] = '=';
						diff[axis_n] = 0;
					}
					loc1 = SerialBuffer + 1;
					position[loc1[0] - 48] = atolong(loc1 + 2);
					while ((loc2 = strstr(loc1 + 2, ",")) != 0) {
						loc1 = loc2 + 1;
						position[loc1[0] - 48] = atolong(loc1 + 2);
					}
					max_diff = 0;
					min_speed = 255;
					max_delay = 0;
					max_soft = 0;
					max_start_delay = 0;
					for (axis_n = 0; axis_n < 6; axis_n++) {
						if (position[axis_n] > Axis[axis_n].position) {
							diff[axis_n] = position[axis_n] - Axis[axis_n].position;
							direction[axis_n] = '+';
						} else if (position[axis_n] < Axis[axis_n].position) {
							diff[axis_n] = Axis[axis_n].position - position[axis_n];
							direction[axis_n] = '-';
						} else {
							diff[axis_n] = 0;
							direction[axis_n] = '=';
						}
						if (diff[axis_n] > max_diff) {
							max_diff = diff[axis_n];
						}
						if (Axis[axis_n].speed < min_speed) {
							min_speed = Axis[axis_n].speed;
						}
						if (Axis[axis_n].delay > max_delay) {
							max_delay = Axis[axis_n].delay;
						}
						if (Axis[axis_n].soft > max_soft) {
							max_soft = Axis[axis_n].soft;
						}
						if (Axis[axis_n].start_delay > max_start_delay) {
							max_start_delay = Axis[axis_n].start_delay;
						}
					}
					max_delay2 = max_delay;
					for (n = 1; n <= max_diff; n++) {
                                        	if (soft_start == 1) {
							if (max_diff < max_soft * 2) {
								max_delay2 = max_start_delay;
							} else {
								if (n < max_soft) {
									max_delay2 = max_delay + (int)((float)(max_soft - n) * (float)(max_start_delay / max_delay2));
								} else if (n > max_diff - max_soft) {
									max_delay2 = max_delay + (int)((float)(max_soft - (float)(max_diff - n)) * (max_start_delay / max_delay2));
								} else {
									max_delay2 = max_delay;
								}
							}
                                                }
						for (axis_n = 0; axis_n < 6; axis_n++) {
							if (direction[axis_n] == '-') {
								stepper_pos(axis_n, '=', sposition[axis_n] - (int)((float)n * ((float)diff[axis_n] / (float)max_diff)), 1);
							} else if (direction[axis_n] == '+') {
								stepper_pos(axis_n, '=', sposition[axis_n] + (int)((float)n * ((float)diff[axis_n] / (float)max_diff)), 1);
							}
						}
						delay_ms(min_speed * -1 + 255);
						delay(max_delay2 * 20 + 2000);
					}
					for (axis_n = 0; axis_n < 6; axis_n++) {
						if (position[axis_n] != Axis[axis_n].position) {
							stepper_pos(axis_n, '=', position[axis_n], 0);
						}
					}
					Uart_Send_Str("OK\n");

				} else if (SerialBuffer[0] == 'O') {
					motor_set(0, 0);
					motor_set(1, 0);
					motor_set(2, 0);
					motor_set(3, 0);
					PORTA = 0;
					PORTC = 0;
					Uart_Send_Str("OK\n");
				} else {
					Uart_Send_Str("ERR\n");
				}
			}
			receive_flag = 0;
		}
	}
}


