
// ####### Config #######
// #define F_CPU               16000000
// #define UART_BAUD_RATE          9600
#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)
#define SERIAL_RECEIVE SIGNAL(SIG_UART_RECV)


#if defined(__AVR_AT90S2313__)
#  define USED_UCR    UCR
#  define USED_UBRR   UBRR
#elif defined(__AVR_ATmega8__)
#  define USED_UCR    UCSRB
#  define USED_UBRR   UBRRL
#elif defined(__AVR_ATmega8515__)
#  define USED_UCR    UCSRB
#  define USED_UBRR   UBRRL
#else
#  error "Don't know what kind of MCU you are compiling for"
#endif


static volatile unsigned char *uart_data_ptr;
static volatile unsigned char uart_counter;
unsigned char *int2str(unsigned char *str, long val);

unsigned char SerialBuffer[20];
unsigned char SerialBuffer_pos = 0;
unsigned char receive_flag = 0;
unsigned char receive_active = 0;

void Uart_Init(void) {
	DDRD &= ~(1<<PD0);
	DDRD |= (1<<PD1);
	PORTD &= ~(1<<PD0);
	PORTD |= (1<<PD1);
	USED_UCR = (1<<RXCIE)|(1<<TXCIE)|(1<<RXEN)|(1<<TXEN);
	USED_UBRR = (unsigned char)UART_BAUD_SELECT;
	sei();
}

SIGNAL(SIG_UART_TRANS) {
	uart_data_ptr++;
	if (--uart_counter) {
		UDR = *uart_data_ptr;
	}
}

void Uart_Send(unsigned char *buf, unsigned char size) {
	if (!uart_counter) {
		uart_data_ptr  = buf;
		uart_counter   = size;
		UDR = *buf;
	}
	while(uart_counter);
}

void Uart_Send_Str(unsigned char *buf) {
	if (!uart_counter) {
		uart_data_ptr  = buf;
		uart_counter   = strlen(buf);
		UDR = *buf;
	}
	while(uart_counter);
}

void Uart_Send_Int(long value) {
	unsigned char txt[9];
	int2str(txt, value);
	Uart_Send_Str(txt);
}

unsigned char *int2str(unsigned char *str, long val) {
	char tmp_str[9];
	int n1 = 0;
	int n2 = 0;
	long value;
	strcpy(tmp_str, str);
	value = val;
	n1 = 0;
	do {
		tmp_str[n1++] = value % 10 + '0';
	} while ((value /= 10) > 0);
	tmp_str[n1++] = 0;
	n2 = strlen(tmp_str);
	str[n2--] = 0;
	for (n1 = 0; n1 < strlen(tmp_str); n1++) {
		str[n2] = tmp_str[n1];
		n2--;
	}
	return str;
}

SERIAL_RECEIVE {
	unsigned char Rx;
	Rx = UDR;
	if (receive_flag < 2) {
		if (Rx == '\n' || Rx == '\r') {
			SerialBuffer_pos = 0;
			receive_flag = 2;
		} else {
			SerialBuffer[SerialBuffer_pos++] = Rx;
			SerialBuffer[SerialBuffer_pos] = 0;
			receive_flag = 1;
		}
	}
}
