
#define STAT_ACTIVE	1
#define STAT_SW1	2
#define STAT_SW2	3
#define STAT_DIRECTION	4

typedef struct {
	long position;
	unsigned char speed;
	unsigned char step;
	unsigned char state;
	unsigned char reverse;
	unsigned char soft;
	unsigned int  delay;
	unsigned int  start_delay;
} StepperAxis;

StepperAxis Axis[6];

const unsigned char Vollschritt_1[] = {
	1 + 8,
	1 + 2,
	4 + 2,
	4 + 8,
	1 + 8,
	1 + 2,
	4 + 2,
	4 + 8
};

const unsigned char Vollschritt_2[] = {
	16 + 128,
	16 + 32,
	64 + 32,
	64 + 128,
	16 + 128,
	16 + 32,
	64 + 32,
	64 + 128
};

const unsigned char Vollschritt_3[] = {
	1 + 8,
	1 + 2,
	4 + 2,
	4 + 8,
	1 + 8,
	1 + 2,
	4 + 2,
	4 + 8
};

const unsigned char Vollschritt_4[] = {
	16 + 128,
	16 + 32,
	64 + 32,
	64 + 128,
	16 + 128,
	16 + 32,
	64 + 32,
	64 + 128
};

// Servo-PWM
TIMER_OVERFLOW0 {
	int n;
	TCNT0 = 0;
	if (receive_flag != 1) {
		if (Axis[4].position > 1500) {
			Axis[4].position = 1500;
		}
		if (Axis[5].position > 1500) {
			Axis[5].position = 1500;
		}
		for (n = -150; n < (int)Axis[4].position || n < (int)Axis[5].position; n++) {
			if (n < (int)Axis[4].position) {
				PORTE |= (1<<PE0);
			} else {
				PORTE &= ~(1<<PE0);
			}
			if (n < (int)Axis[5].position) {
				PORTE |= (1<<PE1);
			} else {
				PORTE &= ~(1<<PE1);
			}
		}
		PORTE &= ~(1<<PE0);
		PORTE &= ~(1<<PE1);
	}
}

void stepper_init(void) {
	DDRA = 255;
	PORTA = 0;
	DDRC = 255;
	PORTC = 0;

	DDRB = 0;
	PORTB = 255;

	DDRE |= (1<<PE0);
	PORTE &= ~(1<<PE0);
	DDRE |= (1<<PE1);
	PORTE &= ~(1<<PE1);

	Axis[0].position = 0;
	Axis[0].speed = 255;
	Axis[0].step = 0;
	Axis[0].state = 0;
	Axis[0].reverse = 0;
	Axis[0].soft = 100;
	Axis[0].delay = 220;
	Axis[0].start_delay = 440;

	Axis[1].position = 0;
	Axis[1].speed = 255;
	Axis[1].step = 0;
	Axis[1].state = 0;
	Axis[1].reverse = 0;
	Axis[1].soft = 100;
	Axis[1].delay = 220;
	Axis[1].start_delay = 440;

	Axis[2].position = 0;
	Axis[2].speed = 255;
	Axis[2].step = 0;
	Axis[2].state = 0;
	Axis[2].reverse = 0;
	Axis[2].soft = 100;
	Axis[2].delay = 180;
	Axis[2].start_delay = 720;

	Axis[3].position = 0;
	Axis[3].speed = 255;
	Axis[3].step = 0;
	Axis[3].state = 0;
	Axis[3].reverse = 0;
	Axis[3].soft = 100;
	Axis[3].delay = 120;
	Axis[3].start_delay = 240;

	Axis[4].position = 0;
	Axis[4].speed = 255;
	Axis[4].step = 0;
	Axis[4].state = 0;
	Axis[4].reverse = 0;
	Axis[4].soft = 120;
	Axis[4].delay = 50;
	Axis[4].start_delay = 100;

	Axis[5].position = 0;
	Axis[5].speed = 255;
	Axis[5].step = 0;
	Axis[5].state = 0;
	Axis[5].reverse = 0;
	Axis[5].soft = 120;
	Axis[5].delay = 50;
	Axis[5].start_delay = 100;

	Timer0_Init(TIMER_CLK_1024);
}

void stepper_speed(unsigned char axis_n, unsigned char speed) {
	Axis[axis_n].speed = speed;
}

void stepper_delay(unsigned char axis_n, unsigned char delay) {
	Axis[axis_n].delay = delay;
}

void stepper_input(void) {
	// Check-Input (Switches)
	if (PINB & (1<<PB0)) {
		Axis[0].state &= ~(1<<STAT_SW1);
	} else {
		Axis[0].state |= (1<<STAT_SW1);
	}
	if (PINB & (1<<PB1)) {
		Axis[0].state &= ~(1<<STAT_SW2);
	} else {
		Axis[0].state |= (1<<STAT_SW2);
	}
	if (PINB & (1<<PB2)) {
		Axis[1].state &= ~(1<<STAT_SW1);
	} else {
		Axis[1].state |= (1<<STAT_SW1);
	}
	if (PINB & (1<<PB3)) {
		Axis[1].state &= ~(1<<STAT_SW2);
	} else {
		Axis[1].state |= (1<<STAT_SW2);
	}
	if (PINB & (1<<PB4)) {
		Axis[2].state &= ~(1<<STAT_SW1);
		Axis[5].state &= ~(1<<STAT_SW1);
	} else {
		Axis[2].state |= (1<<STAT_SW1);
		Axis[5].state |= (1<<STAT_SW1);
	}
	if (PINB & (1<<PB5)) {
		Axis[2].state &= ~(1<<STAT_SW2);
		Axis[5].state &= ~(1<<STAT_SW2);
	} else {
		Axis[2].state |= (1<<STAT_SW2);
		Axis[5].state |= (1<<STAT_SW2);
	}
}

void stepper_output(void) {
	PORTC = Vollschritt_1[Axis[0].step] | Vollschritt_2[Axis[1].step];
	PORTA = Vollschritt_3[Axis[2].step] | Vollschritt_4[Axis[3].step];
}

void stepper_info(unsigned char axis_n) {
	stepper_input();
	Uart_Send_Str("Info: ");
	Uart_Send_Int(axis_n);
	Uart_Send_Str("\n");
	Uart_Send_Str("	Pos = ");
	Uart_Send_Int(Axis[axis_n].position);
	Uart_Send_Str("\n");
	if (Axis[axis_n].state & (1<<STAT_DIRECTION)) {
		Uart_Send_Str("	Dir = +\n");
	} else {
		Uart_Send_Str("	Dir = -\n");
	}
	if (Axis[axis_n].state & (1<<STAT_SW1)) {
		Uart_Send_Str("	Sw 1  = On\n");
	} else {
		Uart_Send_Str("	Sw 1  = Off\n");
	}
	if (Axis[axis_n].state & (1<<STAT_SW2)) {
		Uart_Send_Str("	Sw 2  = On\n");
	} else {
		Uart_Send_Str("	Sw 2  = Off\n");
	}
	Uart_Send_Str("	S = ");
	Uart_Send_Int(Axis[axis_n].speed);
	Uart_Send_Str("\n");
	Uart_Send_Str("	D = ");
	Uart_Send_Int(Axis[axis_n].delay);
	Uart_Send_Str("\n");
}

void stepper_pos(unsigned char axis_n, unsigned char direction, long value, unsigned int mode) {
	long n;
	long steps;
	steps = value;
	Axis[axis_n].state |= (1<<STAT_ACTIVE);
	if (direction == '=') {
		if (steps > Axis[axis_n].position) {
			direction = '+';
			steps = steps - Axis[axis_n].position;
		} else if (steps < Axis[axis_n].position) {
			direction = '-';
			steps = Axis[axis_n].position - steps;
		} else {
			steps = 0;
			return;
		}
	}
	stepper_input();
	for (n = 0; n < steps; n++) {
		if (direction == '+') {
			if (Axis[axis_n].reverse == 1) {
				if (Axis[axis_n].state & (1<<STAT_SW2)) {
					break;
				} else {
					if (Axis[axis_n].step > 0) {
						Axis[axis_n].step--;
					} else {
						Axis[axis_n].step = 7;
					}
					Axis[axis_n].state |= (1<<STAT_DIRECTION);
					Axis[axis_n].position++;
				}
			} else {
				if (Axis[axis_n].state & (1<<STAT_SW2)) {
					break;
				} else {
					if (Axis[axis_n].step < 7) {
						Axis[axis_n].step++;
					} else {
						Axis[axis_n].step = 0;
					}
					Axis[axis_n].state |= (1<<STAT_DIRECTION);
					Axis[axis_n].position++;
				}
			}
		} else if (direction == '-') {
			if (Axis[axis_n].reverse == 1) {
				if (Axis[axis_n].state & (1<<STAT_SW1)) {
					Axis[axis_n].position = 0;
					break;
				} else {
					if (Axis[axis_n].step < 7) {
						Axis[axis_n].step++;
					} else {
						Axis[axis_n].step = 0;
					}
					Axis[axis_n].state &= ~(1<<STAT_DIRECTION);
					Axis[axis_n].position--;
				}
			} else {
				if (Axis[axis_n].state & (1<<STAT_SW1)) {
					Axis[axis_n].position = 0;
					break;
				} else {
					if (Axis[axis_n].step > 0) {
						Axis[axis_n].step--;
					} else {
						Axis[axis_n].step = 7;
					}
					Axis[axis_n].state &= ~(1<<STAT_DIRECTION);
					Axis[axis_n].position--;
				}
			}
		}
		stepper_output();
		if (mode == 0) {
			delay_ms(Axis[axis_n].speed * -1 + 255);
			delay(Axis[axis_n].delay * 20 + 2000);
		}
		stepper_input();
	}
	Axis[axis_n].state &= ~(1<<STAT_ACTIVE);
}

